
angular.module('mobility-app').service('AuthService', function(AppInfo, $localstorage, $q, $http, $timeout) {
	var self = this;

	function expireToken(auth) {

		if (AppInfo.tokenExpiration) {
			$timeout(function() {
				console.log('Token is expired. Removing from localstorage');
				$localstorage.removeAuthorization();
			}, AppInfo.tokenExpiration * 1000);

			return;
		}

		var expiryInSeconds = Number(auth.expires_in);

		// if not set, defaults to 1h
		if (isNaN(expiryInSeconds) || expiryInSeconds < 1) {
			expiryInSeconds = 3600;
		}

		// remove 5 seconds to prevent concurrency issues
		if (expiryInSeconds > 5) {
			expiryInSeconds -= 5;
		}

		// save the expiration time for auto login
		$localstorage.setTokenExpiration(expiryInSeconds);

		$timeout(function() {
			$localstorage.removeAuthorization();
		}, expiryInSeconds * 1000);
	};

	function saveTokens(user, auth) {
		var defer = $q.defer();

		$localstorage.setLastLoggedUser(user);

		// save token and clear after the set expiration time
		$localstorage.setAuthorization(auth.token_type + ' ' + auth.access_token);
		expireToken(auth);

		// store token only for devices, browser does not have secure storage
		if (window.KeyManager)
		{
			// remove unnecessary info from original obj
			var dataToStore = {
				access: auth.access_token,
				refresh: auth.refresh_token
			};

			KeyManager.store(AppInfo.AppId, user, JSON.stringify(dataToStore), function(data) {
				defer.resolve(data);
			},
			function(error) {
				//if (window.cordova) alert('Failed storing session data in secure storage. This session will have a limit time before a new login is necessary.');
				defer.resolve(error); // resolve instead of rejecting
			});
		}
		else {
			// message for devices where KeyManager plugin are disabled
			//if (window.cordova) alert('Unable to store session data in secure storage. This session will have a limit time before a new login is necessary.');
			defer.resolve();
		}

		return defer.promise;
	};

	function retrieveTokens(user) {
		var defer = $q.defer();

		// tokens are only for devices, browser does not have secure storage
		if (window.KeyManager && (ionic.Platform.isAndroid() || AppInfo.useKeyChain))
		{
			KeyManager.retrieve(AppInfo.AppId, user,
			function(data) {
				defer.resolve(JSON.parse(data));
			},
			function(error) {
				defer.resolve(error); // resolve instead of rejecting
			});
		}
		else {
			if ($localstorage.isTokenExpired()) {
				defer.resolve();
			}
			else {
				defer.resolve({
					'authorization': $localstorage.getAuthorization()
				});
			}
		}

		return defer.promise;
	};

	function getNewToken(loginData, defer) {
		var request = {
			url: AppInfo.AuthURL,
			method: 'POST',
			headers: {
				'Content-Type': 'application/x-www-form-urlencoded'
			},
			data: $.param({
				'grant_type': "password",
				'client_id': AppInfo.ClientID,
				'username': loginData.user,
				'password': loginData.password
			})
		};

		$http(request).then(
		function(response) {
			if (response.status == 400 || response.status == 401) {
				defer.reject(response);
			}
			else {
				// wait token to be saved before returning
				saveTokens(loginData.user, response.data).then(function() {
					defer.resolve();
				});
			}
		},
		function(error) {
			console.log('Authentication Error:'); console.dir(error);
			defer.reject(error);
		});
	};

	function refreshToken(token, defer) {
		var request = {
			url: AppInfo.AuthURL,
			method: 'POST',
			headers: {
				'Content-Type': 'application/x-www-form-urlencoded'
			},
			data: $.param({
				'grant_type': "refresh_token",
				'client_id': AppInfo.ClientID,
				'refresh_token': token
			})
		};

		$http(request).then(
		function(response) {
			if (response.status == 400 || response.status == 401) {
				defer.reject(response);
			}
			else {
				// wait token to be saved before returning
				saveTokens($localstorage.getLastLoggedUser(), response.data).then(function() {
					defer.resolve();
				});
			}
		},
		function(error) {
			console.log('Refresh Authentication Error:'); console.dir(error);
			defer.reject(error);
		});
	};

	self.validate = function() {

		if ($localstorage.getAuthorization()) {
			var defer = $q.defer();
			defer.resolve();
			return defer.promise;
		}

		return self.authenticate();
	};

	self.authenticate = function(loginData) {
		var defer = $q.defer();

		if (AppInfo.useMockedData) {
			defer.resolve();
			return defer.promise;
		}

		var user = loginData.user || $localstorage.getLastLoggedUser();

		// retrieve tokens saved to secure storage
		retrieveTokens(user).then(function(data) {
			// if present, get new access token
			if (data && data.refresh) {
				refreshToken(data.refresh, defer)
			}

			// if data is not present, call BE to get new tokens.
			else if (loginData && loginData.user && loginData.password) {
				getNewToken(loginData, defer);
			}

			// if authorization is present
			else if (data && data.authorization) {
				defer.resolve();
			}

			// no token available
			else {
				defer.reject();
			}
		});

		return defer.promise;
	};
});
	